import 'package:flutter/material.dart';

import 'package:flutter_custom_toolbox/flutter_toolbox.dart';

class PageParameters extends StatelessWidget {
  const PageParameters({
    super.key,
    required this.appConfig,
    required this.canBeResumed,
  });

  final ApplicationConfigDefinition appConfig;
  final bool canBeResumed;

  final double separatorHeight = 4.0;

  @override
  Widget build(BuildContext context) {
    // Activity settings (top)
    final List<Widget> blockTop = [];
    for (ApplicationSettingsParameter parameter in appConfig.activitySettings) {
      if (parameter.displayedOnTop) {
        for (List<Widget> itemsLine in buildParametersLines(
          parameter: parameter,
          isEnabled: !canBeResumed,
        )) {
          blockTop.add(Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: itemsLine,
          ));
        }
        blockTop.add(SizedBox(height: separatorHeight));
      }
    }

    // Activity settings (bottom)
    final List<Widget> blockBottom = [];
    for (ApplicationSettingsParameter parameter in appConfig.activitySettings) {
      if (!parameter.displayedOnTop) {
        for (List<Widget> itemsLine in buildParametersLines(
          parameter: parameter,
          isEnabled: !canBeResumed,
        )) {
          blockBottom.add(Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: itemsLine,
          ));
        }
        blockBottom.add(SizedBox(height: separatorHeight));
      }
    }

    // Actions buttons (middle block)
    final List<Widget> blockActions = [];
    if (canBeResumed) {
      // Resume activity
      blockActions.add(AspectRatio(
        aspectRatio: 3,
        child: ActivityButtonResumeSaved(
          onPressed: () {
            appConfig.resumeActivity(context);
          },
        ),
      ));
      // Delete saved activity
      blockActions.add(SizedBox.square(
        dimension: MediaQuery.of(context).size.width / 5,
        child: ActivityButtonDeleteSaved(
          onPressed: () {
            appConfig.deleteCurrentActivity(context);
          },
        ),
      ));
    } else {
      // Start new activity
      blockActions.add(
        AspectRatio(
          aspectRatio: 3,
          child: ActivityButtonStartNew(
            onPressed: () {
              appConfig.startNewActivity(context);
            },
          ),
        ),
      );
    }
    blockActions.add(SizedBox(height: separatorHeight));

    // Global blocks list
    final List<Widget> blocks = [
      Expanded(
        child: ListView(
          children: blockTop,
        ),
      ),
      SizedBox(height: separatorHeight),
    ];

    blocks.addAll(blockActions);

    if (blockBottom.isNotEmpty) {
      blocks.addAll(blockBottom);
    }

    return Column(children: blocks);
  }

  List<List<Widget>> buildParametersLines({
    required ApplicationSettingsParameter parameter,
    required bool isEnabled,
  }) {
    final List<ApplicationSettingsParameterItemValue> items = parameter.values;
    final List<Widget> parameterButtons = [];

    if (items.length <= 1) {
      return [];
    }

    final int itemsPerLine =
        parameter.itemsPerLine != 0 ? parameter.itemsPerLine : items.length;

    // Build linear list
    for (ApplicationSettingsParameterItemValue item in items) {
      final Widget parameterButton = BlocBuilder<ActivitySettingsCubit, ActivitySettingsState>(
        builder: (BuildContext context, ActivitySettingsState activitySettingsState) {
          final ActivitySettingsCubit activitySettingsCubit =
              BlocProvider.of<ActivitySettingsCubit>(context);

          final double displayWidth = MediaQuery.of(context).size.width;
          final double itemWidth = displayWidth / itemsPerLine - 4;

          return SizedBox.square(
            dimension: itemWidth,
            child: parameter.buildParameterItem(
              context: context,
              parameter: parameter,
              itemValue: item,
              size: itemWidth,
              onPressed: isEnabled
                  ? () {
                      activitySettingsCubit.set(parameter.code, item.value);
                    }
                  : null,
            ),
          );
        },
      );

      parameterButtons.add(parameterButton);
    }

    // Split buttons in lines according to items count limit per line
    final List<List<Widget>> lines = [];

    List<Widget> line = [];
    for (var parameterButton in parameterButtons) {
      line.add(parameterButton);
      if (line.length == itemsPerLine) {
        lines.add(line);
        line = [];
      }
    }
    if (line.length != itemsPerLine) {
      lines.add(line);
    }

    return lines;
  }
}
